# -*- encoding: utf-8 -*-
# Template language benchmarks
#
# Objective: Generate a 1000x10 HTML table as fast as possible.
#
# Author: Jonas Borgström <jonas@edgewall.com>

import cgi
import sys
import timeit
from StringIO import StringIO
from genshi.builder import tag
from genshi.template import MarkupTemplate, NewTextTemplate

try:
    from elementtree import ElementTree as et
except ImportError:
    et = None

try:
    import cElementTree as cet
except ImportError:
    cet = None

try:
    import neo_cgi, neo_cs, neo_util
except ImportError:
    neo_cgi = None

try:
    import kid
except ImportError:
    kid = None

try:
    from django.conf import settings
    settings.configure()
    from django.template import Context as DjangoContext
    from django.template import Template as DjangoTemplate
except ImportError:
    DjangoContext = DjangoTemplate = None

try:
    from mako.template import Template as MakoTemplate
except ImportError:
    MakoTemplate = None

try:
    from ptemplate.template import Template as PTemplate
except ImportError:
    PTemplate = None

try:
    from ptemplate.ctemplate import CTemplate
except ImportError:
    CTemplate = None

table = [dict(a=1,b=2,c=3,d=4,e=5,f=6,g=7,h=8,i=9,j=10)
          for x in range(1000)]

genshi_tmpl = MarkupTemplate("""
<table xmlns:py="http://genshi.edgewall.org/">
<tr py:for="row in table">
<td py:for="c in row.values()" py:content="c"/>
</tr>
</table>
""")

genshi_tmpl2 = MarkupTemplate("""
<table xmlns:py="http://genshi.edgewall.org/">$table</table>
""")

genshi_text_tmpl = NewTextTemplate("""
<table>
{% for row in table %}<tr>
{% for c in row.values() %}<td>$c</td>{% end %}
</tr>{% end %}
</table>
""")

if DjangoTemplate:
    django_tmpl = DjangoTemplate("""
    <table>
    {% for row in table %}
    <tr>{% for col in row.values %}{{ col|escape }}{% endfor %}</tr>
    {% endfor %}
    </table>
    """)

    def test_django():
        """Djange template"""
        context = DjangoContext({'table': table})
        django_tmpl.render(context)

if MakoTemplate:
    mako_tmpl = MakoTemplate("""
<table>
  % for row in table:
    <tr>
      % for col in row.values():
        <td>${ col | h  }</td>
      % endfor
    </tr>
  % endfor
</table>
""")
    def test_mako():
        """Mako Template"""
        mako_tmpl.render(table=table)

if PTemplate:
    import cgi
    escape = lambda field: cgi.escape(str(field))
    PTemplate.converters.update({"h": escape})
    ptmpl = PTemplate(template="""\
<table>
{#rows}
    <tr>
{#columns}
        <td>{col!h}
{/columns}
    </tr>
{/rows}
</table>
""")
    data = {
        "rows": [{"columns": [{"col": v} for v in r.values()]} for r in table],
    }
    def test_ptemplate():
        """PTemplate"""
        ptmpl.render(data)

if CTemplate:
    import cgi
    escape = lambda field: cgi.escape(str(field))
    CTemplate.converters.update({"h": escape})
    ctmpl = CTemplate(template="""\
<table>
{{#rows}}
    <tr>
{{#columns}}
        <td>{{col!h}}
{{/columns}}
    </tr>
{{/rows}}
</table>
""")
    data = {
        "rows": [{"columns": [{"col": v} for v in r.values()]} for r in table],
    }
    def test_ctemplate():
        """CTemplate"""
        ctmpl.render(data)

def test_genshi():
    """Genshi template"""
    stream = genshi_tmpl.generate(table=table)
    stream.render('html', strip_whitespace=False)

def test_genshi_text():
    """Genshi text template"""
    stream = genshi_text_tmpl.generate(table=table)
    stream.render('text')

def test_genshi_builder():
    """Genshi template + tag builder"""
    stream = tag.TABLE([
        tag.tr([tag.td(c) for c in row.values()])
        for row in table
    ]).generate()
    stream = genshi_tmpl2.generate(table=stream)
    stream.render('html', strip_whitespace=False)

def test_builder():
    """Genshi tag builder"""
    stream = tag.TABLE([
        tag.tr([
            tag.td(c) for c in row.values()
        ])
        for row in table
    ]).generate()
    stream.render('html', strip_whitespace=False)

if kid:
    kid_tmpl = kid.Template("""
    <table xmlns:py="http://purl.org/kid/ns#">
    <tr py:for="row in table">
    <td py:for="c in row.values()" py:content="c"/>
    </tr>
    </table>
    """)

    kid_tmpl2 = kid.Template("""
    <html xmlns:py="http://purl.org/kid/ns#">$table</html>
    """)

    def test_kid():
        """Kid template"""
        kid_tmpl.table = table
        kid_tmpl.serialize(output='html')

    if cet:
        def test_kid_et():
            """Kid template + cElementTree"""
            _table = cet.Element('table')
            for row in table:
                td = cet.SubElement(_table, 'tr')
                for c in row.values():
                    cet.SubElement(td, 'td').text=str(c)
            kid_tmpl2.table = _table
            kid_tmpl2.serialize(output='html')

if et:
    def test_et():
        """ElementTree"""
        _table = et.Element('table')
        for row in table:
            tr = et.SubElement(_table, 'tr')
            for c in row.values():
                et.SubElement(tr, 'td').text=str(c)
        et.tostring(_table)

if cet:
    def test_cet(): 
        """cElementTree"""
        _table = cet.Element('table')
        for row in table:
            tr = cet.SubElement(_table, 'tr')
            for c in row.values():
                cet.SubElement(tr, 'td').text=str(c)
        cet.tostring(_table)

if neo_cgi:
    def test_clearsilver():
        """ClearSilver"""
        hdf = neo_util.HDF()
        for i, row in enumerate(table):
            for j, c in enumerate(row.values()):
                hdf.setValue("rows.%d.cell.%d" % (i, j), cgi.escape(str(c)))

        cs = neo_cs.CS(hdf)
        cs.parseStr("""
<table><?cs
  each:row=rows
?><tr><?cs each:c=row.cell
  ?><td><?cs var:c ?></td><?cs /each
?></tr><?cs /each?>
</table>""")
        cs.render()


def run(which=None, number=10):
    tests = ['test_builder', 'test_genshi', 'test_genshi_text',
             'test_genshi_builder', 'test_mako', 'test_kid', 'test_kid_et',
             'test_et', 'test_cet', 'test_clearsilver', 'test_django',
             'test_ptemplate', 'test_ctemplate']

    if which:
        tests = filter(lambda n: n[5:] in which, tests)

    for test in [t for t in tests if hasattr(sys.modules[__name__], t)]:
        t = timeit.Timer(setup='from __main__ import %s;' % test,
                         stmt='%s()' % test)
        time = t.timeit(number=number) / number

        if time < 0.00001:
            result = '   (not installed?)'
        else:
            result = '%16.2f ms' % (1000 * time)
        print '%-35s %s' % (getattr(sys.modules[__name__], test).__doc__, result)


if __name__ == '__main__':
    which = [arg for arg in sys.argv[1:] if arg[0] != '-']

    if '-p' in sys.argv:
        import hotshot, hotshot.stats
        prof = hotshot.Profile("template.prof")
        benchtime = prof.runcall(run, which, number=1)
        stats = hotshot.stats.load("template.prof")
        stats.strip_dirs()
        stats.sort_stats('time', 'calls')
        stats.print_stats()
    else:
        run(which)



import unittest
try:
    import unittest2
except ImportError:
    unittest2 = False

import warnings

class BaseTest(unittest.TestCase):

    def skipTest(self, reason):
        pass

if unittest2:
    class BaseTest(unittest2.TestCase):
        
        def __init__(self, methodName="runTest"):
            super(BaseTest, self).__init__(methodName)
            self.addTypeEqualityFunc(str, 'assertMultiLineEqual')

        def skipTest(self, reason):
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                super(BaseTest, self).skipTest(reason)

class TemplateTest(BaseTest):
    cls = None

    def assertProduces(self, input, expect, kwargs={}, args=()):
        output = self.cls(template=input).render(kwargs)
        self.assertEqual(expect, output)

class ModifierTest(TemplateTest):
    
    def assertProduces(self, input, expect, kwargs={}, args=(), converters={}):
        templater = self.cls(template=input)
        templater.formatter.converters.update(converters)
        output = templater.render(kwargs)
        self.assertEqual(expect, output)



"""\
:mod:`ptemplate.template` -- advanced string templating
-------------------------------------------------------

:mod:`ptemplate.template` provides a :class:`Template`, a thin interface on top
:mod:`ptemplate.formatter` that is more useful for typical templating tasks.
"""

__license__ = """Copyright (c) 2010 Will Maier <will@m.aier.us>

Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""

from ptemplate.formatter import Formatter
from ptemplate.util import logger

__all__ = ["Template"]

class Template(object):
    """A templater.

    :class:`Template` wraps :class:`ptemplate.formatter.Formatter` with a
    _`Buffet`-compatible _`interface`. In addition to the standard Buffet
    arguments (*extra_vars_func*, *options*), the constructor accepts a
    *template* string. This string is the template that will be rendered later
    (by a call to :meth:`render`).

    .. _Buffet:     http://pypi.python.org/pypi/Buffet/
    .. _interface:  http://docs.turbogears.org/1.0/TemplatePlugins
    """
    options = {}
    preprocessor = None
    """Template preprocessor callable.

    This callable should accept the template string as its sole argument. Its
    output should be another template string. Use this facility to translate
    foreign template syntaxes into something
    :class:`ptemplate.formatter.Formatter` can understand.
    """
    converters = {}
    """A dictionary of object converters.

    These converters will be passed to the template's
    :class:`ptemplate.formatter.Formatter` instance.
    """
    
    def __init__(self, extra_vars_func=None, options=None, template=''):
        self.log = logger(__name__, self)
        self.options = options
        self.template = template
        self.formatter = Formatter()
        """The template's formatter.

        The formatter performs the actual templating work and should
        have a :meth:`ptemplate.formatter.Formatter.format` method and a
        :attr:`ptemplate.formatter.Formatter.converters` dictionary. The
        converters dictionary will be updated with any converters specified in
        the Template.
        """

        self.formatter.converters.update(self.converters)

    def render(self, data, format="html", fragment=False, template=None):
        """Render the template using *data*.

        The *format*, *fragment* and *template* arguments are ignored. Instead,
        :class:`Template` uses :attr:`template` as the template, passing it to
        :attr:`preprocessor` if necessary. It then expands the template (using
        :attr:`formatter`) and returns the result as a string.
        """
        template = self.template
        preprocessor = getattr(self, "preprocessor", None)
        if callable(preprocessor):
            template = preprocessor(template)
        self.formatter.converters.update(self.converters)
        return self.formatter.format(template, **data)

    def transform(self, info, template): # pragma: nocover
        """Render the output to Elements.

        Required by Buffet; not supported.
        """
        raise NotImplementedError

    def load_template(self, templatename): # pragma: nocover
        """Find a template specified in Python 'dot' notation.

        Required by Buffet; not supported.
        """
        raise NotImplementedError



from tests import TemplateTest

from ptemplate.ctemplate import CTemplate

class TestCTemplate(TemplateTest):
    cls = CTemplate

    def test_basic(self):
        self.assertProduces("{{foo}}", "bar", {"foo": "bar"})

    # The following are ported from Google's template_unittest.cc,
    # starting around line 400.

    def test_weird_syntax_triple_nested(self):
        self.skipTest("needs better-than-naive preprocessor")
        self.assertProduces("hi {{{! VAR {{!VAR} }} lo", "hi { lo")

    def test_weird_syntax_triple(self):
        self.assertProduces("fn(){{{BI_NEWLINE}} x=4;{{BI_NEWLINE}}}",
            "fn(){\n x=4;\n}")

    def test_weird_syntax_tons_of_brackets(self):
        self.skipTest("needs better-than-naive preprocessor")
        self.assertProduces("{{{{{{VAR}}}}}}}}", "{{{{}}}}}}")

    def test_comment(self):
        self.assertProduces("hi {{!VAR}} lo", "hi  lo")

    def test_comment_nested(self):
        self.assertProduces("hi {{!VAR {VAR} }} lo", "hi  lo")

    def test_comment_nested_broken(self):
        self.skipTest("needs better-than-naive preprocessor")
        self.assertProduces("hi {{! VAR {{!VAR} }} lo", "hi  lo")

    # Skipping TestSetMarkerDelimiters; no plans to support that feature.

class TestCTemplateVariables(TemplateTest):
    cls = CTemplate

    def setUp(self):
        self.data = {}
        self.input = "hi {{VAR}} lo"

    def test_variable_nodata(self):
        self.assertProduces(self.input, "hi  lo", self.data)

    def test_variable_simple(self):
        self.data["VAR"] = "yo"
        self.assertProduces(self.input, "hi yo lo", self.data)

    def test_variable_similar(self):
        self.data["VAR"] = "yoyo"
        self.data["VA"] = "noyo"
        self.data["VAR "] = "noyo2"
        self.data["var"] = "noyo3"
        self.assertProduces(self.input, "hi yoyo lo", self.data)

    # Skipping TestVariableWithModifiers; modifiers should come eventually.

class TestCTemplateSection(TemplateTest):
    cls = CTemplate

    def setUp(self):
        self.data = {}
        # Note: The original test uses whitespace stripping.
        #input = "boo!\nhi {{#SEC}}lo{{#SUBSEC}}jo{{/SUBSEC}}{{/SEC}} bar"
        self.input = "boo!hi {{#SEC}}lo{{#SUBSEC}}jo{{/SUBSEC}}{{/SEC}} bar\n"

    def test_section_nodata(self):
        self.assertProduces(self.input, "boo!hi  bar\n", self.data)

    def test_section_outerdata(self):
        self.data["SEC"]  = [{}]
        self.assertProduces(self.input, "boo!hi lo bar\n", self.data)

    def test_section_innerdata(self):
        self.data["SEC"]  = [{}]
        self.data["SUBSEC"] = [{}]
        self.assertProduces(self.input, "boo!hi lojo bar\n", self.data)

    # Skipping TestSectionSeparator; separators probably won't be supported in
    # the language itself.

    # Skipping TestInclude; include files probably wont' be supported in the
    # language itself (though it'd be easy).
    # Skipping TestIncludeWithModifiers; see above.
    # Skipping TestRecursiveInclude; see above.

class TestCTemplateInheritance(TemplateTest):
    cls = CTemplate

    def setUp(self):
        self.data = {
            "FOO": "foo",
            "SEC": [{
                "SEC": [{
                    "blah": "blah",
                }]
            }]
        }
        self.input = "{{FOO}}{{#SEC}}{{FOO}}{{#SEC}}{{FOO}}{{/SEC}}{{/SEC}}\n"

    def test_inheritance(self):
        self.assertProduces(self.input, "foofoofoo\n", self.data)

    def test_inheritance_two_levels(self):
        self.data["SEC"][0]["FOO"] = "bar"
        self.assertProduces(self.input, "foobarbar\n", self.data)

    def test_inheritance_three_levels(self):
        self.data["SEC"][0]["FOO"] = "bar"
        self.data["SEC"][0]["SEC"][0]["FOO"] = "baz"
        self.assertProduces(self.input, "foobarbaz\n", self.data)

    # Skipping TestTemplateString; not relevant.
    # Skipping TestExpand; not relevant.
    # Skipping TestAnnotation; not relevant.

class TestCTemplateRegTest(TemplateTest):
	
	def setUp(self):
		# Defined in template_regtest.cc lines 224..
		self.data = {
			"HEAD": "   This is the head   ",
			"FOOTER_BAR_TEXT": [{
				"BODY": "Should never be shown",
				"HOME_LINK": "<b>Time to go home!</b>", # html_escape
				"ADVERTISE_LINK": "<b>Be advertiser #2</b>",
				"ABOUT_GOOGLE_LINK": "<A HREF=/>About Google!</A>",
			}],
			"PROMO_LICENSING_SECTION": [{
				"PROMO_LICENSING_LINK": "<A HREF='foo'>",
			}],
			"TRIM_LINE_COLOR": "Who cares?",
			"TRIM_LINE_HEIGHT": 10,
			"MODIFIED_BY_GOOGLE": 2005,
			"MSG_copyright": "&copy; Google Inc. (all rights reserved)",
			"CLOSING_DIV_SECTION": [{}],
			"LATENCY_PREFETCH_URL": "/latency",
			"LATENCY_PREFETCH": [{}],
			"JAVASCRIPT_FOOTER_SECTION": [{
				"FAST_NEXT_JAVASCRIPT": [{}],
			}],
		}



"""Add included modules here.
"""
import os

# Add included module names to __all__.
__all__ = []
project = os.path.basename(os.path.dirname(__file__))
ext = project + "._ext"

name, module = None, None
for name in __all__:
    try:
        module = __import__(name)
    except ImportError:
        module = __import__('.'.join((ext, name)), fromlist=[ext])
    locals()[name] = module

del(ext, module, name, project)



"""\
:mod:`ptemplate.formatter` -- extended string formatter
-------------------------------------------------------

This module extends the advanced string formatter (:pep:`3101`) available in
Python versions greater than 2.5. In addition to regular variable substitution
and formatting, :class:`Formatter` supports sections and in-template comments.
"""

__license__ = """Copyright (c) 2010 Will Maier <will@m.aier.us>

Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""

import string
from collections import namedtuple

from ptemplate.util import logger

__all__ = ["Formatter", "Section", "Token"]

Section = namedtuple("Section", "name tokens data scopes conversion format")
"""A template section.

Constructor arguments should be passed as keywords and include:

*name* is the name of the section (not including the section marker).

*tokens* is a list of :class:`Token` instances that belong in the section.

*data* is a template data dictionary applicable to the section.

*scopes* is a list of data dictionaries that should be searched successively
when resolving variables in the section.

*conversion* is a conversion string. If not None, it will be passed with the
output of the section to :meth:`Formatter.convert_field`.

*format* is a format string. If not None, it will be passed with the output of
the section to :meth:`Formatter.format_field`.
"""
Token = namedtuple("Token", "text field fieldname marker spec conversion")
"""A template token.

Constructor arguments should be passed as keywords and include:

*text* is the literal text preceding the token.

*field* is the name of the field (or None), not including the marker indicator.

*fieldname* is the full name of the field (or None), including the marker
indicator.

*marker* is a marker type registered in :attr:`Formatter.markers`.

*spec* is a format string passed with the token's contents to
:meth:`Formatter.format_field`.

*conversion* is a conversion string passed with the token's contents to
:meth:`Formatter.convert_field`.
"""

class Formatter(string.Formatter):
    """A string formatter.

    :class:`Formatter` extends the advanced string formatter described in
    :pep:`3101` and implemented in Python versions greater than 2.5. It
    provides hooks that simplify token conversion (:attr:`converters`) and
    recognize special template variables (:attr:`markers`). Since it uses
    :meth:`str._formatter_parser` (implemented in C in the standard interpreter)
    to parse the input string, it should also perform relatively well.
    """
    converters = {}
    """A dictionary of converter functions keyed by conversion strings.

    If a token's conversion string matches a key in this dictionary, :meth:`convert_field`
    will use the converter instead of the usual string conversion.

    .. note::
        
        Keys in this dictionary must be one character in length (to match
        :meth:`str._formatter_parser`'s expectations for format flags).
    """
    markers = {
        '#': "startsection",
        '/': "endsection",
        '%': "comment",
    }
    """A dictionary mapping marker indicators to marker type names.

    The extended syntax supported by :class:`Formatter` is composed of markers
    within normal template variables. Markers registered here can trigger
    extended behavior in :meth:`_vformat`.
    """
    markerlen = 1
    """The length of a marker indicator."""

    def __init__(self, *args, **kwargs):
        super(Formatter, self).__init__(*args, **kwargs)
        self.log = logger(__name__, self)

    def vformat(self, string, args, kwargs):
        """Format *string* according to data in *args* and *kwargs*.

        Unlike :meth:`string.Formatter.vformat`, this method does not track (or
        complain about) unused arguments.
        """
        return self._vformat(string, args, kwargs)

    def _vformat(self, string, args, kwargs):
        """Do the real string formatting.

        This method tokenizes *string* (:meth:`tokenize`) and passes the stream
        of tokens to :meth:`formatsection`, returning the templated string.
        """
        tokens = self.tokenize(string)
        return self.formatsection(tokens, kwargs)

    def tokenize(self, string):
        """Tokenize a template *string*.

        :meth:`tokenize` parses *string* using :meth:`string.Formatter.parse`
        (which in turn relies on :meth:`str._formatter_parser`) and yields
        :class:`Token` instances using the parsed data.
        """
        for text, field, spec, conversion in self.parse(string):
            fieldname = field
            marker = None
            if field and len(field) >= self.markerlen:
                indicator = field[:self.markerlen]
                if indicator in self.markers:
                    marker = self.markers[indicator]
                    field = field[self.markerlen:]
                
            yield Token(text, field, fieldname, marker, spec, conversion)

    def formatsection(self, tokens, data, scopes=[]):
        """Format a section of a token stream according to *data*.

        :meth:`formatsection` builds a formatted string from the iterable *tokens*.
        If it encounters a section in the stream, it creates a new stream and
        passes the section (and everything in it) to another invocation of
        :meth:`formatsection`, adding its output to the formatted string. This
        continues recursively until there are no more sections and the stream of
        tokens is finished. Then, the formatted result is returned.

        When a section is completed, its output will be passed to
        :meth:`convert_field` and :meth:`format_field` if the :attr:`Section.conversion`
        or :attr:`Section.format` attributes were defined, respectively.
        """
        sections = []
        result = []
        depth = 0

        for token in tokens:
            section = sections and sections[-1] or Section(None, [], {}, [], None, None)
            text = token.text

            # Short circuit parsing if...
            if depth == 0 and section.name == token.field and token.marker == "endsection":
                # ...we're closing the topmost section (depth=0); render the
                # subsection and continue.
                section.tokens.append(Token(text, None, None, None, None, None))
                _data, _ = self.get_field(token.field, (), [data] + scopes)
                for d in _data:
                    content = self.formatsection(section.tokens, d, [data] + scopes)
                    if section.conversion:
                        content = self.convert_field(content, section.conversion)
                    if section.format:
                        content = self.format_field(content, section.format)
                    result.append(content)
                sections.pop()
                continue
            elif section.name is not None:
                # ...we're in a section; add our token to the section's list and
                # continue.
                section.tokens.append(token)

                # Track depth (but not the subsections themselves).
                if token.marker == "startsection": depth += 1
                elif token.marker == "endsection": depth -= 1
                continue

            # Always add the token's text to the result. Since the parser produces
            # tokens with preceding text, this has to happen early.
            if text:    
                result.append(text)

            if token.marker == "startsection":
                section = Section(name=token.field, tokens=[], data=data,
                    scopes=scopes, conversion=token.conversion, format=token.spec)
                sections.append(section)
            elif token.marker == "endsection": # pragma: nocover
                # Already handled.
                pass
            elif token.field is not None:
                # Perform the usual string formatting on the field.
                obj, _ = self.get_field(token.field, (), [data] + scopes)
                obj = self.convert_field(obj, token.conversion)
                spec = super(Formatter, self)._vformat(token.spec, (), data, (), 2)
                result.append(self.format_field(obj, spec))

        return ''.join(result)

    def get_value(self, field, args, scopes):
        """Look up the value of *field* in *scopes*.

        *scopes* is a list of data dictionaries associated with each successive
        parent of the current section. :meth:`get_value` searches for a key
        matching *field* in each of these dictionaries, in order. If no match is
        found, an empty string is returned. Otherwise, the key's value (a list
        of data dictionaries or a string) is returned.
        """
        _scopes = list(scopes)
        value = ''
        while _scopes and value == '':
            scope = _scopes.pop(0)
            value = scope.get(field, '')

        return value

    def convert_field(self, value, conversion):
        """Convert a field *value* according to a *conversion* specification.

        If *conversion* is registered in :attr:`converters`, the *value* is
        passed to the matching converter and the output is returned. Otherwise,
        the *value* and *conversion* specification are passed to
        :meth:`string.Formatter.convert_field`.
        """
        converter = self.converters.get(conversion, None)
        if callable(converter):
            value = converter(value)
        else:
            value = super(Formatter, self).convert_field(value, conversion)
        return value



from django.template import Library, Node, resolve_variable
from django.utils.html import escape

register = Library()

def greeting(name):
    return 'Hello, %s!' % escape(name)
greeting = register.simple_tag(greeting)



from tests import ModifierTest, TemplateTest

from ptemplate.template import Template

class TestTemplate(TemplateTest):
    cls = Template

    def test_standard_template(self):
        input = "foo {bar}"
        data = {"bar": 1}
        self.assertProduces(input, input.format(**data), data)

    def test_nonexistent_keys(self):
        self.assertProduces("{dne}", "")

    def test_markers_comment(self):
        self.assertProduces("{%this is a comment}", "")

    def test_markers_comment_space(self):
        self.assertProduces("{% this is a comment}", "")

    def test_sections_simple(self):
        data = {
            "section": [
                {"a":"b"}, {}, {},
            ]
        }
        input = '\n'.join([
            "before the section",
            "{#section}",
            "inside the section",
            "some more text",
            "{/section}",
            "and after the section"])
        output = '\n'.join([
            "before the section\n",
            "inside the section",
            "some more text\n",
            "inside the section",
            "some more text\n",
            "inside the section",
            "some more text\n",
            "and after the section"])
        self.assertProduces(input, output, data)

    def skip_test_sections_nested(self):
        data = {
            "outer": [
                {"inner": [ {} ]},
            ],
        }
        input = '\n'.join([
            "beginning",
            "{#outer}",
            "outer start",
            "{#inner}",
            "inner contents",
            "{/inner}",
            "outer finish",
            "{/outer}"
            "end"])
        output = '\n'.join([
            "beginning\n",
            "outer start\n",
            "inner contents\n",
            "outer finish",
            "end"])
        self.assertProduces(input, output, data)

    def test_sections_nodata(self):
        data = {}
        input = '\n'.join([
            "beginning",
            "{#section}",
            "middle",
            "{/section}",
            "end"])
        output = '\n'.join([
            "beginning\n",
            "end"])
        self.assertProduces(input, output, data)

    def test_smorgasbord(self):
        data = {
            "foo": "the foo",
            "results": [
                {"thisis": "letters", "one": "a", "two": "b"},
                {"thisis": "digits", "one": "1", "two": "2"},
            ]
        }
        input = '\n'.join([
            "{% this is a comment}",
            "la la la",
            "{#results}",
            "this is: {thisis}",
            "this is from a different scope: {foo}",
            "one: {one}",
            "two: {two}",
            "and some more text",
            "{/results}",
            "and something after the section"])
        output = '\n'.join([
            "\nla la la\n",
            "this is: letters",
            "this is from a different scope: the foo",
            "one: a",
            "two: b",
            "and some more text\n",
            "this is: digits",
            "this is from a different scope: the foo",
            "one: 1",
            "two: 2",
            "and some more text\n",
            "and something after the section",
        ])
        self.assertProduces(input, output, data)

class TestModifierHTMLEscape(ModifierTest):
    cls = Template

    def setUp(self):
        import webob
        self.converters = {"h": webob.html_escape}

    def test_modifier_variable_html_escape(self):
        input = """<div class="foo">{foo!h}</div>"""
        data = {"foo": """<a href="http://www.example.com/foo">foo</a>"""}
        output = """<div class="foo">&lt;a href=&quot;http://www.example.com/foo&quot;&gt;foo&lt;/a&gt;</div>"""
        self.assertProduces(input, output, data, converters=self.converters)

    def test_modifier_section_html_escape(self):
        input = """<div class="foo">{#foo!h}inside{bar}{/foo}</div>"""
        data = {"foo": [{"bar": "<p>one!</p>"}, {"bar": "<p>two!</p>"}]}
        output = """<div class="foo">inside&lt;p&gt;one!&lt;/p&gt;inside&lt;p&gt;two!&lt;/p&gt;</div>"""
        self.assertProduces(input, output, data, converters=self.converters)

class TestFormatting(TemplateTest):
    cls = Template
    
    def test_variable_formatting(self):
        input = """->{foo:^15}<-"""
        data = {"foo": "center this"}
        output = "->  center this  <-"
        self.assertProduces(input, output, data)
    
    def test_section_formatting(self):
        input = """->{#section:^5}|{foo}|{/section}<-"""
        data = {"section": [{"foo": "a"},{"foo": "b"},{"foo": "c"}]}
        output = "-> |a|  |b|  |c| <-"
        self.assertProduces(input, output, data)



"""\
:mod:`ptemplate.ctemplate` -- ctemplate-like interface
------------------------------------------------------

This module implements a templating interface similar Google's `ctemplate
<http://google-ctemplate.googlecode.com/>`_ with a few important exceptions:

* templates may not change the field delimiter
* modifiers are marked with '!'
* comments are marked with '%'
* the templater does not strip whitespace (except by modifiers)
* includes are not supported
* pragmas/macros are not supported
* separator sections are not supported

Like ctemplate, :class:`CTemplate` expands a string template to match the
structure of a dictionary (using :class:`ptemplate.template.Template`). Data dictionaries
may contain either lists of other data dictionaries or single values (string,
integer, float, etc). Plain variables are substituted according to the usual
Python string formatting rules (see :pep:`3101`). Sections (fields preceded by
'#') are expanded once for each data dictionary contained in the corresponding
list.
"""

__license__ = """Copyright (c) 2010 Will Maier <will@m.aier.us>

Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""

from ptemplate.template import Template
from ptemplate.util import logger

__all__ = ["CTemplate"]

class CTemplate(Template):
    """A (somewhat) ctemplate-compatible templater.

    Incompatibilities with Google's ctemplate are documented in
    :mod:`ptemplate.ctemplate`. Construction of a :class:`CTemplate` instance
    is the same as with :class:`ptemplate.template.Template`.
    """
    globals = {
        "BI_NEWLINE": '\n',
        "BI_SPACE": ' ',
    }
    """A global data dictionary.

    Template variables that don't match keys in the main data dictionary may
    match global keys as a last resort. By default, :attr:`globals` contains
    BI_NEWLINE and BI_SPACE, which match the newline and space characters,
    respectively (as in ctemplate).
    """

    def __init__(self, *args, **kwargs):
        super(CTemplate, self).__init__(*args, **kwargs)
        self.log = logger(__name__, self)

    def preprocessor(self, input):
        """Convert Google's ctemplate syntax.

        Since ctemplate and :mod:`ptemplate.template` are quite similar
        internally, a simple translation can make a document (mostly) legible.
        At the moment, the preprocessor only converts the marker indicators
        ('{{' and '}}') and comment character ('!').
        """
        input = input.replace("{", "{{")
        input = input.replace("{{{{", "{")
        input = input.replace("}", "}}")
        input = input.replace("}}}}", "}")

        input = input.replace("{!", "{%")

        return input

    def render(self, data, format="html", fragment=False, template=None):
        """Render the template.

        Here, :class:`CTemplate` adds the :attr:`globals` dictionary to the *data*
        dictionary before calling :meth:`ptemplate.template.Template.render`.
        """
        globals = self.globals.copy()
        globals.update(data)
        return super(CTemplate, self).render(globals)



try:
    import setuptools
except ImportError:
    from distribute_setup import use_setuptools
    use_setuptools()
from setuptools import setup, find_packages

import sys, os

libdir = "lib"

sys.path.insert(0, libdir)

import ptemplate as pkg

setup_options = {
    "name": pkg.__project__,
    "version": pkg.__version__,
    "description": pkg.__description__,
    "long_description": pkg.__doc__,
    "classifiers": pkg.__classifiers__,
    "keywords": pkg.__keywords__,
    "author": pkg.__author__,
    "author_email": pkg.__author_email__,
    "url": pkg.__url__,
    "packages": find_packages(libdir),
    "package_dir": {"": libdir},
    "include_package_data": True,
    "zip_safe": False,
    "install_requires": pkg.__requires__,
    "entry_points": """
        # -*- Entry points: -*-
        [python.templating.engines]
        ptemplate = ptemplate.template:Template
        ctemplate = ptemplate.ctemplate:CTemplate
    """,
    "test_suite": "tests",
}

# Enable 2to3 on 3.x.
if sys.version_info >= (3, 0):
    setup_options["use_2to3"] = True

setup(**setup_options)



"""\
ptemplate is a lightweight templating engine for Python, heavily inspired by
Google's `ctemplate <http://google-ctemplate.googlecode.com/>`_ language.
Like ctemplate, ptemplate strongly separates the application's logic and
presentation. At its heart, ptemplate is an extension of Python's advanced
string formatting facility -- it extends the interpreter's built-in formatters
to provide a fast, expressive, content-agnostic templating system.
"""

__project__ = "ptemplate"
__version__ = "0.1.2"
__pkg__ = "ptemplate"
__description__ = "data-based templating"
__author__ = "Will Maier"
__author_email__ = "willmaier@ml1.net"
__url__ = "http://packages.python.org/ptemplate"

# See http://pypi.python.org/pypi?%3Aaction=list_classifiers.
__classifiers__ = """\
Development Status :: 3 - Alpha
Environment :: Web Environment :: Buffet
Intended Audience :: Developers
License :: OSI Approved :: MIT License
Natural Language :: English
Programming Language :: Python
Topic :: Text Processing
""".splitlines()
__keywords__ = "template text json data"

__requires__ = []

# The following is modeled after the ISC license.
__copyright__ = """\
Copyright (c) 2010 Will Maier <willmaier@ml1.net>

Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
"""

__todo__ = """\
"""
